
BODY {
	FONT-FAMILY: verdana, arial, sans-serif
}

a:link.selfref, a:visited.selfref {
 color: #555 !important;
 }

a:link, a:visited {
 font-family: sans-serif;
 color:#000099;
 text-decoration: none;
}

a:active, a:hover {
 color:#990000;
 text-decoration: underline;
 }


TABLE {
	FONT-SIZE: 9px;
	border-spacing: 2px;
}

TD {
	border: 1px solid #555555;
}

TD.hbreak {
	border: none;
}

.navBar {
 Font-weight: normal; Font-size: 12px;
}

TD.itemList {
 	Font-weight: normal; Font-size: 12px; FONT-FAMILY: arial, sans-serif;
	padding:2px;
	border: none;
}

TD.itemList a {
	line-height: 1.4em;
}

DIV.navBar {
 font-size: 8pt;
 padding-top:4px; 
 color:#222222;
}

DIV.navBar a {
 color:#222222;
}

FONT.navBarCurrent {
    padding: 4px; font-weight: bold; color: #FFFFFF; background-color:#990000;
}

TD.reportHeader {
	 BACKGROUND: #eeeeee; 
	PADDING: 5px; 
	FONT-WEIGHT: bold; 
	FONT-SIZE: 12px; 
	BORDER: #555555 1px solid;
	border: none;
}

.titleText {
        font-weight: bold; 
        font-size: normal; 
        font-family: sans-serif; 
        color:#990000; 
        line-height: 2em;
        border: 1px solid #aaaaaa;
        padding-left:4px;
        padding-right:4px;
        padding-top:2px;
        padding-bottom:2px;
}

TD.titleText {
	 BACKGROUND: #eff7ff; PADDING-left: 5px;
	background: #dddddd;
        font-size: 10pt; 
}

TD.headerStats {
	border: none;
}

TD.headerStats table td {
	border: none;
	color:#555555;
}
TD.graphHeaderLeft {
         BACKGROUND: #cccccc; 
	BORDER: #777777 1px solid;
	color: #990000;
	font-size: 8pt;
}
TD.graphHeader {
         BACKGROUND: #cccccc; 
	BORDER-BOTTOM: #777777 1px solid; 
	BORDER-TOP: #777777 1px solid;
	BORDER-RIGHT: #777777 1px solid;
	color: #990000;
	font-size: 8pt;
}

TD.graphItem {
	BACKGROUND: #eeeeee; 
	BORDER-BOTTOM: #777777 1px solid; 
	BORDER-LEFT: #777777 1px solid; 
	BORDER-RIGHT: #777777 1px solid; 
	PADDING-LEFT: 15px; 
	PADDING-RIGHT: 15px; 
	FONT-WEIGHT: bold; FONT-SIZE: 12px; 
}

TD.graphPercent {
	font-size:8pt;
         BACKGROUND: #eeeeee; 
	border-bottom: 1px solid #777777;
	border-right: 1px solid #777777;
}
TD.graphBarLeft {
	font-size:8pt;
	BACKGROUND: #eeeeee; 
	border-bottom: 1px solid #777777;
	border-right: 1px solid #777777;
} 
TD.graphBar {
	BACKGROUND: white;
	border: none;
	padding:2px;
	width: 80%;
} 

TD.hspacer {
	BORDER-BOTTOM:#dcdcdc 1px solid; BORDER-TOP:#dcdcdc 1px solid;
}

TABLE.barGraph {
	WIDTH: 100%;
}

TD.empty {
	FONT-SIZE: 2px; BACKGROUND: #c0c0c0; BORDER:#9c9c9c 1px solid;
}
TD.fullcover {
	FONT-SIZE: 2px; BACKGROUND: #00df00; BORDER:#9c9c9c 1px solid;
}
TD.covered {
	FONT-SIZE: 2px; BACKGROUND: #009900; 
	border:1px solid black;
}

TD.uncovered {
	FONT-SIZE: 2px; BACKGROUND: #990000; 
	border:none;
	border:1px solid black;
}
TD.tabText {
	PADDING-RIGHT: 12px; 
}

DIV.timestamp {
	FONT-SIZE: 10px; font-weight: normal; padding-left: 1px;
	color:#555555;
}
SPAN.srcFile {
	FONT-SIZE: 9px
}

TABLE.srcView {
	FONT-SIZE: 12px; 
	BORDER:#555555 1px solid;
}

TD.lineCount {
	border: 1px solid #999999;
	BACKGROUND: #F0F0F0; 
	PADDING-RIGHT: 3px;
}
TD.lineCountHilight {
	border: 1px solid #999999;
	BACKGROUND: #C8C8F0; 
	PADDING-RIGHT: 3px;
}

SPAN.Hilight {
  BACKGROUND: #C8C8F0;
}

SPAN.srcHilight pre {
  BACKGROUND: #ddaaaa;
}

TD.coverageCount {
	border: 1px solid #999999;
	BACKGROUND: #F0F0F0; 
	PADDING-RIGHT: 3px;
}

TD.coverageCountHilight {
	border: 1px solid #999999;
  	BACKGROUND: #F0C8C8; 
	PADDING-RIGHT: 3px;
}

TD.srcLine {
	border: none;
   WIDTH: 100%	
}
TD.srcLineClassStart {
	border: none;
   WIDTH: 100%; BORDER-TOP:#dcdcdc 1px solid; FONT-WEIGHT: bold;	
}

PRE.srcLine {
  BACKGROUND: #ffffff; MARGIN-TOP: 0px; MARGIN-BOTTOM: 0px; 
}

TD.footer {
FONT-SIZE: 9px;
background-color:#dddddd;
}

TD.footer td {
  color:#777777;
}
