body {
 background: #fff;
 color: #000;
 letter-spacing:1px;
 }

a {
	font-weight: normal;
}

table {
	border: none;
	margin: 0px;
	padding: 0px;
}

.app h3 {
        font-family: sans-serif;
	border: solid 1px #999999;
        padding:6px;
        background-color:#dddddd;
	color:#990000;
	font-size:14pt;
	margin: 1pt;
	margin-top: 6pt;
	margin-left: -10px;
}

.app h3 a {
	font-size:14pt;
	font-stretch:expanded;
}

.app h3 a:before {
	content: "[ ";
}

.app h3 a:after {
	content: " ]";
}



.app h4 {
        font-family: sans-serif;
        border:#bbbbbb solid 1px;
        padding:2px;
        background-color:#eeeeee;
	color:#990000;
	margin-left: -5px;
	padding-left: 10px;
}

.app h4 a {
	font-size:8pt;
}

.app h4 a:before {
	content: "...";
}

.app h4 a:after {
	content: "...";
}

.app th {
	background-color: #cccccc;
	color: #990000;
        font-family: sans-serif;
        font-size: 8pt;
        border: solid #777777 1px;
	padding: 4px;
	letter-spacing: normal;
	font-weight:normal;
}

.app td {
        font-family: sans-serif;
        font-size: 8pt;
        border: solid #777777 1px;
	padding: 4px;
}

div#banner {
	border: none;
}

code {
	color: #990000;
}

.b td { 
 background: #dddddd;
 color: #000;
 border: solid #555555 1px;
 font-family: sans-serif;
 }

.a td { 
 background: #eeeeee;
 color: #000;
 font-family: sans-serif;
 }

div#banner {
/*
 border-top: 1px solid #369;
 border-bottom: 1px solid #003;
*/
 }

#bodycol div.h3 p {
        max-width: 5in;
	margin-left: 20px;
	line-height: 1.6em;
	font-size: 8pt;
	color:#777777;
	letter-spacing:normal;
}

#banner, #banner td { 
 background: white;
 color: #fff;
 }

#leftcol {
 background: #eeeeee;
 color: #000;
 border-right: 1px solid #777777;
 border-bottom: 1px solid #777777;
/*
 border-bottom: 1px solid #aaa;
 border-top: 1px solid #fff;
*/
}

#navcolumn {
 background: #eeeeee;
 color: #000;
 border-right: none;
 border-bottom: none;
 border-top: none;
 min-width:1.5in;
 margin-left:10px;
 }

#navcolumn div strong {
  color:#990000;
  padding: 4px;
  line-height:20pt;
  border-bottom: 1px solid #bbbbbb;
  font-weight: normal;
  letter-spacing: normal;
}

#navcolumn div {
  margin-top:10px;
}

#navcolumn div div {
  margin-bottom:10px;
}

#breadcrumbs {
    border: 1px solid #777777;
}

#breadcrumbs {
 background-color: #dddddd;
 }

#breadcrumbs td {
	border: none;
	color: #777777;
	padding-left: 1em;
}

#source {
 background-color: #fff;
 color: #000;
 border-right: 1px solid #888; 
 border-left: 1px solid #888; 
 border-top: 1px solid #888; 
 border-bottom: 1px solid #888; 
 margin-right: 7px;
 margin-left: 7px;
 margin-top: 1em;
 }

pre {
 margin-right: 7px;
 margin-left: 7px;
 }

#footer {
  background: #eeeeee;
  border: 1px solid #777777;
  margin-left: 0px;
  margin-right: 0px;
  padding-top: 3px;
  padding-bottom: 3px;
  padding-left: 1em;
  color:#777777;
}

#footer table {
}

#footer table tr td {
  border: none;
  padding: 0px;
  margin: 4px;
}

a:link, #breadcrumbs a:visited, #navcolumn a:visited, .tasknav a:visited {
 font-family: sans-serif;
 color:#000099;
 text-decoration: none;
 }

a:active, a:hover, #leftcol a:active, #leftcol a:hover {
 font-family: sans-serif;
 color:#990000;
 text-decoration: none;
 }

a:link.selfref, a:visited.selfref {
 color: #555 !important;
 }

.app a:link, .app a:visited {
 font-family: sans-serif;
 color:#000099;
 text-decoration: none;
}

.app a:active, .app a:hover {
 color:#990000;
 text-decoration: underline;
 }

h3, h4 {
 margin-top: 1em;
 margin-bottom: 0;
 }
