/* Javadoc style sheet */

/* Define colors, fonts and other style attributes here to override the defaults  */

/* Page background color */
body { background-color: #FFFFFF }


a:link.selfref, a:visited.selfref {
 color: #555 !important;
 }

a:link, a:visited {
 font-family: sans-serif;
 color:#000099;
 text-decoration: none;
}

a:active, a:hover {
 color:#990000;
 text-decoration: underline;
 }



font.FrameItemFont {
 font-size:10pt;
}

a:link.selfref, a:visited.selfref {
 font-family: sans-serif;
 color: #555 !important;
 }

.a td {
 background: #ddd;
 color: #000;
 font-family: sans-serif;
 }

h2 {
	font-family: sans-serif;
	border:solid 1px #999999;
	padding:10px;
	background-color:#EEEEEE;
}

table {
	border: none;
	border-spacing: 2px;
}


tr.TableHeadingColor td {
 	font-family: sans-serif;
 	font-size: 9pt;
	border: 1px solid #555555;
}

tr.TableRowColor td {
 	font-family: sans-serif;
 	font-size: 9pt;
	border: 1px solid #999999;
	cell-padding: 2px;
}

td.NavBarCell1 table {
	border: none;
}

td.NavBarCell1 {
	border: none;
}

td.NavBarCell2 {
	border: 4px green;
}

dt {
 	font-family: sans-serif;
 	font-size: 10pt;
}

dt b {
	color:#990000;
}

dd {
	font-size:10pt;
	margin-top:4px;
	margin-bottom:4px;
}

dd code {
	color:#333333;
	font-size:9pt;
	font-size:normal;
}

/* Table colors */
.TableHeadingColor     { background: #cccccc; color:#990000} 
.TableSubHeadingColor  { background: #bbbbbb; color:black;}
.TableRowColor         { background: #EEEEEE } 

/* Font used in left-hand frame lists */
.FrameTitleFont   { font-weight: 900; font-size: normal; font-family: sans-serif; color:#FFFFFF }
.FrameHeadingFont { 
	font-weight: bold; 
	font-size: normal; 
	font-family: sans-serif; 
	color:#990000; 
	line-height: 2em;
	border: 1px solid #aaaaaa;
	padding-left:4px;
	padding-right:4px;
	padding-top:2px;
	padding-bottom:2px;
}
.FrameItemFont    { font-size: normal; font-family: sans-serif; color:#FFFFFF }

/* Example of smaller, sans-serif font in frames */
/* .FrameItemFont  { font-size: 10pt; font-family: Helvetica, Arial, sans-serif } */

/* Navigation bar fonts and colors */

.NavBarCell1    { 
	background-color:#ddd; 
	border:none;
	padding: 2px;
}

.NavBarCell1Rev { 
	background-color:#990000;
	border:none;
	padding: 2px;
}

.NavBarFont1    { 
	font-family: Arial, Helvetica, sans-serif; 
	font-size: 8pt;
	color:#000000;
}

.NavBarFont1Rev { 
	font-family: Arial, Helvetica, sans-serif; 
	font-size: 8pt;
	color:#FFFFFF;
}

.NavBarCell2    { font-family: Arial, Helvetica, sans-serif; background-color:#FFFFFF; border:none;}
.NavBarCell3    { font-family: Arial, Helvetica, sans-serif; background-color:#FFFFFF; border:none;}
